import os
import sys
import time
import subprocess
import signal
import gettext
import pwd
from statvfs import *
from gettext import gettext as _
from optparse import OptionParser
from pkg_resources import resource_filename
from backup_manager.core import *

log = None

def backup_needed (config):
    if not config.last_backup:
        return False
    
    days = (time.time () - config.last_backup) / 86400 
    if config.backup_schedule == SCHEDULE_NEVER:
        return False
    elif config.backup_schedule == SCHEDULE_HOURLY:
        return days > 3
    elif config.backup_schedule == SCHEDULE_DAILY:
        return days > 7
    elif config.backup_schedule == SCHEDULE_WEEKLY:
        return days > 30
    elif config.backup_schedule == SCHEDULE_MONTHLY:
        return days > 90
    else:
        raise BackupManagerException (_("Unknown schedule type '%s'") % sch)

def start_applet ():
    pid = os.fork ()
    if pid != 0:
        return pid
   
    os.setsid ()
    os.chdir ("/")
    os.umask (0)
  
    pids = get_session_pids (pwd.getpwuid (os.getuid ())[0])
    if len (pids) == 0:
        print "No active sessions. skipping applet launch."
        os._exit (0)

    new_env = get_env_from_pid (pids[0])
    os.execvpe ("python", ["pyton", resource_filename (__name__, "applet.py")], new_env)

def backup_now (config):
    applet_pid = start_applet ()
    ck = None
   
    try:
        ck = block_suspend ()
    except BackupManagerException, e:
        print >> sys.stderr, str (e)

    try:
        get_backup_manager ().backup (config)
    except BackupManagerException, e:
        os.kill (applet_pid, signal.SIGINT)
        notify_user_err (_("Backup Error"), str (e))
        sys.exit (1)
    else:
        notify_user (_("Backup"), _("Your data has been backed up"), expire=False)
    finally:
        if ck:
            unblock_suspend (ck)
            
def check_uuid_foreach_session (uuid):
    p = subprocess.Popen (["ps -C gnome-session -o pid=,user="],
                          stdout=subprocess.PIPE, close_fds=True, shell=True)
    output = p.stdout.read ().strip ()
    p.wait ()
    
    for line in output.split ("\n"):
        pid, user = line.strip ().split ()
        bm = subprocess.Popen (["su", user, "-c", "backup-manager --check-uuid=%s --session-pid=%s" % (uuid, pid)],
                                close_fds=True)
        bm.wait ()

def check_uuid (config, uuid, session_pid):
    if config.device_uuid != uuid:
        return False

    new_env = get_env_from_pid (session_pid) 
    for k, v in new_env.iteritems ():
        os.putenv (k ,v)
    
    import gtk
    import gtk.glade
    
    gtk.gdk.threads_init ()
    widgets = gtk.glade.XML (resource_filename (__name__, "backup.glade"))
    md = widgets.get_widget ("backup_message_dialog")

    res = md.run ()
    md.hide ()
    md.destroy ()
    return res == 1
        
def main ():
    global log
    gettext.bindtextdomain ("backup-manager", "/usr/share/locale")
    gettext.textdomain ("backup-manager")
    
    parser = OptionParser ()
    parser.add_option ("-k", "--check", dest="check", action="store_true",
                       default=False, help="Check to see if we should backup now.")
    parser.add_option ("-c", "--config", dest="config", 
                       default=None, help="Configuration file to use.")
    parser.add_option ("-n", "--now", dest="now", action="store_true",
                       default=False, help="Backup now")
    parser.add_option ("-a", "--device-added", dest="device", action="store_true", 
                       default=False, help="A storage device was added.  See if it's a backup device.")
    parser.add_option ("-u", "--check-uuid", dest="check_uuid", 
                       default=None, help="Check to see if our backup device matches UUID and backup if necessary.")
    parser.add_option ("-s", "--session-pid", dest="session_pid", 
                       default=None, help="Provide the pid of the session manager. Only used with --check-uuid.")
    parser.add_option ("-m", "--mount-delay", dest="mount_delay", action="store_true", 
                       default=False, help="Wait up to 30 seconds for the backup device to be mounted")
    (options, args) = parser.parse_args ()

    config_file = get_config_location ()
    config = Config ()

    if options.config and os.path.exists (options.config):
        config.load (options.config)
    elif os.path.exists (config_file):
        config.load (config_file)
    elif options.device:
        pass
    else:
        parser.print_help ()
        if options.now:
            notify_user_err (_("Backup Error"), _("Unable to find backup configuration"), expire=False)
        sys.exit (1)

    if options.check:
        log = get_logger (redirect=True)
        if backup_needed (config):
            backup_now (config)
        else:
            print _("Backup not necessary. Exiting.")
    elif options.device:
        uuid = os.getenv ("HAL_PROP_VOLUME_UUID")
        if not uuid:
            print >> sys.stderr, "HAL_PROP_VOLUME_UUID is not set."
            sys.exit (1)
        check_uuid_foreach_session (uuid)
    elif options.now:
        log = get_logger (redirect=True)
        if options.mount_delay:
            for i in xrange (0, 5):
                time.sleep (5.0)
                vol_info = get_volume_info ()
                if config.device_uuid and vol_info.has_key (config.device_uuid):
                    break
        backup_now (config)
    elif options.check_uuid:
        if config.prompt_on_attach == False:
            print "Prompt on attach is disabled. Exiting..."
            sys.exit (0)

        log = get_logger (redirect=True)
        if check_uuid (config, options.check_uuid, options.session_pid):
            os.execlp ("backup-manager", "backup-manager", "-m", "--now")
    else:
        parser.print_help ()
        
if __name__ == "__main__":
    try:
        main ()
    except Exception, e:
        global log
        if not log:
            log = get_logger (redirect=True)
        
        log.error (str (e))
        sys.exit (1)
