import os
import sys
import datetime
import gtk
import gtk.glade
import gettext
from gettext import gettext as _
from optparse import OptionParser
from pkg_resources import resource_filename
from backup_manager.core import *

log = None

class RestoreWindow ():
    def __init__ (self, config):
        self._config = config
        self._manager = get_backup_manager ()
        self._files_mdl = gtk.ListStore (gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        self._search_mdl = gtk.ListStore (gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        self._ampm_mdl = gtk.ListStore (gobject.TYPE_STRING)
        self._progress_bar = gtk.ProgressBar ()
        self._checking = False
        self._path = None
        
        self._widgets = gtk.glade.XML (resource_filename (__name__, "backup.glade"))
        self._window = self._widgets.get_widget ("restore_window")
        self._search_added_img = self._window.render_icon (gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
        self._cancel_button = self._widgets.get_widget ("restore_cancel_button")
        self._restore_files_window = self._widgets.get_widget ("restore_files_window")
        self._restore_search_window = self._widgets.get_widget ("restore_search_window")
        self._select_files_button = self._widgets.get_widget ("restore_select_files_button")
        self._restore_close_button = self._widgets.get_widget ("restore_close_button")
        self._add_file_button = self._widgets.get_widget ("restore_add_file_button")
        self._add_folder_button = self._widgets.get_widget ("restore_add_folder_button")
        self._restore_search_button = self._widgets.get_widget ("restore_search_button")
        self._remove_button = self._widgets.get_widget ("restore_remove_button")
        self._files_view = self._widgets.get_widget ("restore_files_view")
        self._search_view = self._widgets.get_widget ("search_view")
        self._restore_all_button = self._widgets.get_widget ("restore_all_button")
        self._restore_manual_button = self._widgets.get_widget ("restore_manual_button")
        self._restore_hour_spin = self._widgets.get_widget ("restore_hour_spin")
        self._restore_minute_spin = self._widgets.get_widget ("restore_minute_spin")
        self._restore_time_combo = self._widgets.get_widget ("restore_time_combo")
        self._restore_button = self._widgets.get_widget ("restore_button")
        self._restore_chooser_button = self._widgets.get_widget ("restore_chooser_button")
        self._restore_search_entry = self._widgets.get_widget ("restore_search_entry")
        self._restore_search_add_button = self._widgets.get_widget ("restore_search_add_button")
        self._restore_search_remove_button = self._widgets.get_widget ("restore_search_remove_button")
        self._restore_do_search_button = self._widgets.get_widget ("restore_do_search_button")
        self._location_hbox = self._widgets.get_widget ("location_hbox")
        self._main_content = self._widgets.get_widget ("restore_content")
        self._calendar = self._widgets.get_widget ("restore_calendar")
        self._restore_now_button = self._widgets.get_widget ("restore_now_button")
        self._tod_label = self._widgets.get_widget ("tod_label")
        
        self._window.connect ("delete-event", lambda w, e: gtk.main_quit ())
        self._cancel_button.connect ("clicked", lambda b: gtk.main_quit ())
        self._restore_files_window.connect ("delete-event", lambda w, e: w.hide () or True)
        self._restore_search_window.connect ("delete-event", lambda w, e: w.hide () or True)
        self._select_files_button.connect ("clicked", self._files_button_clicked) 
        self._restore_close_button.connect ("clicked", self._restore_files_close)
        self._add_file_button.connect ("clicked", self._add_file)
        self._add_folder_button.connect ("clicked", self._add_folder)
        self._remove_button.connect ("clicked", self._remove_folder)
        self._restore_search_button.connect ("clicked", self._show_search)
        self._restore_button.connect ("clicked", self._restore)
        self._restore_all_button.connect ("toggled", self._data_selection_changed)
        self._restore_now_button.connect ("toggled", self._restore_now_toggled) 
        self._restore_search_entry.connect ("changed", self._search_entry_changed)
        self._restore_do_search_button.connect ("clicked", self._start_search)
        self._restore_search_entry.connect ("activate", self._start_search)
        self._restore_search_add_button.connect ("clicked", self._search_add_clicked)
        self._restore_search_remove_button.connect ("clicked", self._search_remove_clicked)

        self._files_view.set_model (self._files_mdl)
        self._files_view.append_column (gtk.TreeViewColumn (None, gtk.CellRendererText (), text=0))
        self._files_view.get_selection ().connect ("changed", self._files_view_changed)
        
        self._search_view.set_model (self._search_mdl)
        self._search_view.append_column (gtk.TreeViewColumn (None, gtk.CellRendererPixbuf (), pixbuf=0))
        self._search_view.append_column (gtk.TreeViewColumn (None, gtk.CellRendererText (), text=1))
        self._search_view.get_selection ().connect ("changed", self._search_view_changed)
        self._search_view.get_selection ().set_mode (gtk.SELECTION_MULTIPLE)

        self._restore_do_search_button.set_sensitive (False)
        self._restore_search_add_button.set_sensitive (False)
        self._restore_search_remove_button.set_sensitive (False)

        self._restore_all_button.set_active (True)
        self._restore_time_combo.set_model (self._ampm_mdl)
        self._ampm_mdl.append (["AM"])
        self._ampm_mdl.append (["PM"])
        cell = gtk.CellRendererText ()
        self._restore_time_combo.pack_start (cell, True)
        self._restore_time_combo.add_attribute (cell, 'text', 0)
        self._restore_time_combo.set_active (0)
        self._backup_device_chooser = gtk.FileChooserButton (_("Select Backup Device"))
        self._backup_device_chooser.set_action (gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)

        if self._config.device_uuid or self._config.fuse_type:
            try:
                self._path = get_backup_path (config)
                label = gtk.Label(self._path)
                label.set_alignment (0.0, 0.5)
                self._location_hbox.pack_start (label)
            except BackupManagerException, e:
                print >> sys.stderr, str (e)
                self._set_storage_device_if_available ()
                self._location_hbox.pack_start (self._backup_device_chooser)
        else:
            self._set_storage_device_if_available ()
            self._location_hbox.pack_start (self._backup_device_chooser)

        self._restore_now_toggled (self._restore_now_button)
        self._window.show_all ()

    def _set_storage_device_if_available (self):
        vol_info = get_volume_info ()
        for k, v in vol_info.iteritems ():
            if v[3]:
                self._backup_device_chooser.set_current_folder (v[0])
                return

    def _restore_files_close (self, button):
        self._restore_files_window.hide ()
    
    def _files_button_clicked (self, b):
        self._restore_files_window.set_transient_for (self._window)
        self._restore_files_window.show_all ()
    
    def _data_selection_changed (self, b):
        val = self._restore_all_button.get_active ()
        self._select_files_button.set_sensitive (not val) 

    def _restore_now_toggled (self, tb):
        act = tb.get_active ()
        self._calendar.set_sensitive (not act)
        self._tod_label.set_sensitive (not act)
        self._restore_hour_spin.set_sensitive (not act)
        self._restore_minute_spin.set_sensitive (not act)
        self._restore_time_combo.set_sensitive (not act)

    def _show_search (self, button):
        self._restore_search_window.set_transient_for (self._window)
        self._restore_search_window.show_all ()
        self._restore_search_window.present ()

    def _search_entry_changed (self, entry):
        val = self._restore_search_entry.get_text ().strip () == ""
        self._restore_do_search_button.set_sensitive (not val)

    def _search_view_changed (self, view):
        sel = self._search_view.get_selection ()
        mdl, lst = sel.get_selected_rows ()
        self._restore_search_add_button.set_sensitive (len (lst) != 0)
        self._restore_search_remove_button.set_sensitive (len (lst) != 0)

    def _search_add_clicked (self, b):
        sel = self._search_view.get_selection ()
        mdl, lst = sel.get_selected_rows ()

        for p in lst:
            iter = mdl.get_iter (p)
            mdl.set_value (iter, 0, self._search_added_img)
            restore_file = mdl.get_value (iter, 1)

            found = False
            for row in self._files_mdl:
                if row[0] == restore_file:
                    found = True
                    break

            if not found:
                self._files_mdl.append ([restore_file, mdl.get_value (iter, 2), mdl.get_value (iter, 3)])
                    
    def _search_remove_clicked (self, b):
        def check_match (mdl, path, iter, data):
            v = mdl.get_value (iter, 0)
            if v == data[0]:
                data[1].append (path)
            return False

        sel = self._search_view.get_selection ()
        mdl, lst = sel.get_selected_rows ()
        
        for p in lst:
            iter = mdl.get_iter (p)
            mdl.set_value (iter, 0, None)
            restore_file = mdl.get_value (iter, 1)

            paths = []
            self._files_mdl.foreach (check_match, (restore_file, paths))
            paths.reverse ()

            for p in paths:
                self._files_mdl.remove (self._files_mdl.get_iter (p))
    
    def _start_search (self, w):
        self._restore_search_window.set_title (_("Searching..."))
        self._restore_do_search_button.set_sensitive (False)
        self._restore_search_entry.set_sensitive (False)
        self._restore_search_add_button.set_sensitive (False)
        self._restore_search_remove_button.set_sensitive (False)
        self._search_view.set_sensitive (False)
        self._search_mdl.clear ()
        
        t = BackupManagerThread (target=self._do_search)
        t.set_callbacks (finish_cb=self._end_search, error_cb=self._end_search)
        t.start ()

    def _do_search (self):
        res = self._manager.restore_search (self._config, self._restore_search_entry.get_text ())
        if not res:
            return

        gtk.gdk.threads_enter ()
        for pretty_name, name, timestr in res:
            self._search_mdl.append ([None, pretty_name, name, timestr])
        gtk.gdk.threads_leave ()

    def _end_search (self):
        gtk.gdk.threads_enter ()
        self._restore_search_window.set_title (_("Search"))
        self._restore_do_search_button.set_sensitive (True)
        self._restore_search_entry.set_sensitive (True)
        self._search_view.set_sensitive (True)
        gtk.gdk.threads_leave ()

    def _add_file (self, button):
        fc = gtk.FileChooserDialog (title=_("Add File"), parent=self._window,
                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
        fc.set_select_multiple (True)

        if self._path:
            fc.set_current_folder (self._path)
        else:
            fl = self._backup_device_chooser.get_current_folder ()
            fc.set_current_folder (fl)

        resp = fc.run ()
        if resp == gtk.RESPONSE_OK:
            for fn in fc.get_filenames ():
                self._files_mdl.append ([fn, fn, None])
        fc.destroy ()

    def _add_folder (self, button):
        fc = gtk.FileChooserDialog (title=_("Add Folder"), parent=self._window,
                                    action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
        fc.set_select_multiple (True)
        
        if self._path:
            fc.set_current_folder (self._path)
        else:
            fl = self._backup_device_chooser.get_current_folder ()
            fc.set_current_folder (fl)
        
        resp = fc.run ()
        if resp == gtk.RESPONSE_OK:
            for fn in fc.get_filenames ():
                self._files_mdl.append ([fn, fn, None])
        fc.destroy ()

    def _remove_folder (self, button):
        sel = self._files_view.get_selection ()
        (mdl, iter) = sel.get_selected ()
        if iter:
            mdl.remove (iter)

    def _files_view_changed (self, view):
        sel = self._files_view.get_selection ()
        (mdl, iter) = sel.get_selected ()
        self._remove_button.set_sensitive (iter != None)

    def _restore (self, b):
        if self._restore_now_button.get_active ():
            self._restore_point = datetime.datetime.now ().replace (microsecond=0)
        else:
            (year, month, day) = self._calendar.get_date ()
            am = self._restore_time_combo.get_active () == 0
            hour = self._restore_hour_spin.get_value_as_int ()
            hour = (hour + 12) % 24 if not am else hour
            minute = self._restore_minute_spin.get_value_as_int ()
            self._restore_point = datetime.datetime (year, month, day, hour, minute, 0)
        
        self._backup_path = self._path if self._path else self._backup_device_chooser.get_filename ()
        self._restore_path = self._restore_chooser_button.get_filename ()

        self._window.set_title (_("Restoring data..."))
        self._main_content.set_sensitive (False)
        self._main_content.pack_start (self._progress_bar)
        self._progress_bar.set_text (_("Restoring data..."))
        self._window.show_all ()

        all_files = self._restore_all_button.get_active ()
        t = BackupManagerThread (target=self._restore_all if all_files else self._restore_specific)
        t.set_callbacks (finish_cb=self._finish_cb, error_cb=self._error_cb)
        t.start ()
        
        self._checking = True
        gobject.timeout_add (500, lambda: self._progress_bar.pulse () or self._checking)

    def _restore_all (self):
        self._manager.verify_restore_data (self._backup_path, self._restore_path, None, self._restore_point)
        self._manager.restore (self._backup_path, self._restore_path, self._restore_point)

    def _restore_specific (self):
        files = []
        for f in self._files_mdl:
            if f[2] == None:
                files.append ((f[1], self._restore_point.isoformat ()))
            else:
                files.append ((f[1], f[2]))
        
        self._manager.verify_restore_data (self._backup_path, self._restore_path, files, self._restore_point)
        self._manager.restore_specific (self._backup_path, self._restore_path, files)

    def _finish_cb (self):
        self._config.save ()
        notify_user (_("Restore Complete"), _("Your data has been restored"), expire=False)
        gtk.main_quit ()

    def _error_cb (self, msg):
        gtk.gdk.threads_enter ()
        self._checking = False
        self._window.set_title (_("Restore!"))
        self._main_content.set_sensitive (True)
        self._main_content.remove (self._progress_bar)
        md = gtk.MessageDialog (parent=self._window, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK)
        md.set_icon (self._window.get_icon ())
        md.set_markup (_("<b>Failed to restore data:</b>\n") + msg)
        md.run ()
        md.destroy ()
        gtk.gdk.threads_leave ()

def main ():
    gettext.bindtextdomain ("backup-manager", "/usr/share/locale")
    gettext.textdomain ("backup-manager")
    
    parser = OptionParser ()
    parser.add_option ("-c", "--config", dest="config",
                       default=None, help="Configuration file to use")
    (options, args) = parser.parse_args ()
    
    gtk.gdk.threads_init ()

    config_file = get_config_location ()
    config = Config ()
    
    if options.config and os.path.exists (options.config):
        config.load (options.config)
    elif os.path.exists (config_file):
        config.load (config_file)
    else:
        config.config_file = config_file
   
    global log
    log = get_logger (redirect=True)
    
    rw = RestoreWindow (config)
    gtk.main ()

if __name__ == "__main__":
    try:
        main ()
    except Exception, e:
        global log
        if not log:
            log = get_logger (redirect=True)
        
        log.error (str (e))
        sys.exit (1)
