import os
import os.path
import gobject
import string
import gtk
import gtk.glade
import gettext
from gettext import gettext as _
from optparse import OptionParser
from pkg_resources import resource_filename
from backup_manager.core import *

log = None

class SettingsWindow ():
    def __init__ (self, config):
        self._config = config
        self._manager = get_backup_manager ()
        self._files_mdl = gtk.ListStore (gobject.TYPE_STRING)
        self._ampm_mdl = gtk.ListStore (gobject.TYPE_STRING)
        self._exceptions_mdl = gtk.ListStore (gobject.TYPE_STRING)
        self._progress_bar = gtk.ProgressBar ()
        self._checking = False

        self._widgets = gtk.glade.XML (resource_filename (__name__, "backup.glade"))
        self._window = self._widgets.get_widget ("settings_window")
        self._select_files_window = self._widgets.get_widget ("select_files_window")
        self._backup_combo = self._widgets.get_widget ("backup_combo")
        self._files_button = self._widgets.get_widget ("files_button")
        self._type_frame = self._widgets.get_widget ("type_frame")
        self._storage_frame = self._widgets.get_widget ("storage_frame")
        self._schedule_frame = self._widgets.get_widget ("schedule_frame")
        self._chooser_button = self._widgets.get_widget ("chooser_button")
        self._all_data_radio = self._widgets.get_widget ("all_data_radio")
        self._include_radio = self._widgets.get_widget ("include_radio")
        self._cancel_button = self._widgets.get_widget ("close_button")
        self._apply_button = self._widgets.get_widget ("apply_button")
        self._add_file_button = self._widgets.get_widget ("add_file_button")
        self._add_folder_button = self._widgets.get_widget ("add_folder_button")
        self._remove_button = self._widgets.get_widget ("remove_button")
        self._select_close_button = self._widgets.get_widget ("select_close_button")
        self._ex_add_file_button = self._widgets.get_widget ("ex_add_file_button")
        self._ex_add_folder_button = self._widgets.get_widget ("ex_add_folder_button")
        self._ex_remove_button = self._widgets.get_widget ("ex_remove_button")
        self._files_view = self._widgets.get_widget ("selected_files_view")
        self._exceptions_view = self._widgets.get_widget ("exceptions_view")
        self._main_content = self._widgets.get_widget ("main_content")
        self._hour_spin_button = self._widgets.get_widget ("hour_spin_button")
        self._minute_spin_button = self._widgets.get_widget ("minute_spin_button")
        self._ampm_combo = self._widgets.get_widget ("ampm_combo")
        self._backup_now_button = self._widgets.get_widget ("backup_now_button")
        self._prompt_on_attach_button = self._widgets.get_widget ("device_attached_button")
        self._purge_container = self._widgets.get_widget ("purge_container")
        self._purge_data_button = self._widgets.get_widget ("purge_data_button")
        self._purge_spin_button = self._widgets.get_widget ("purge_spin_button")
        self._purge_combo = self._widgets.get_widget ("purge_combo")
        self._save_label = self._widgets.get_widget ("save_label")

        self._window.connect ("delete-event", lambda w, e: gtk.main_quit ())
        self._select_files_window.connect ("delete-event", lambda w, e: w.hide () or True)
        self._files_button.connect ("clicked", self._files_button_clicked) 
        self._cancel_button.connect ("clicked", lambda b: gtk.main_quit ())
        self._apply_button.connect ("clicked", self._save)
        self._include_radio.connect ("toggled", lambda b: self._files_button.set_sensitive (b.get_active ()))
        self._add_file_button.connect ("clicked", self._add_file)
        self._add_folder_button.connect ("clicked", self._add_folder)
        self._remove_button.connect ("clicked", self._remove_folder)
        self._select_close_button.connect ("clicked", self._select_files_close)
        self._ex_add_file_button.connect ("clicked", self._add_file_exception)
        self._ex_add_folder_button.connect ("clicked", self._add_folder_exception)
        self._ex_remove_button.connect ("clicked", self._remove_exception)
        self._backup_combo.connect ("changed", self._frequency_changed)
        self._backup_now_button.connect ("toggled", self._backup_now_toggled)
        self._purge_data_button.connect ("toggled", self._purge_data_toggled)

        self._files_button.set_sensitive (self._include_radio.get_active ())
        self._all_data_radio.set_active (True)

        self._files_view.set_model (self._files_mdl)
        self._files_view.append_column (gtk.TreeViewColumn (None, gtk.CellRendererText (), text=0))
        self._files_view.get_selection ().connect ("changed", self._files_view_changed)
        self._exceptions_view.set_model (self._exceptions_mdl)
        self._exceptions_view.append_column (gtk.TreeViewColumn (None, gtk.CellRendererText (), text=0))
        self._exceptions_view.get_selection ().connect ("changed", self._exceptions_view_selection_changed)

        mdl = gtk.ListStore (gobject.TYPE_STRING)
        for k, v in sorted (schedule_map.iteritems ()):
            mdl.append ([v])
        self._backup_combo.set_model (mdl)
        cell = gtk.CellRendererText ()
        self._backup_combo.pack_start (cell, True)
        self._backup_combo.add_attribute (cell, 'text', 0)
        self._backup_combo.set_active (2)
        
        self._ampm_combo.set_model (self._ampm_mdl)
        self._ampm_mdl.append (["AM"])
        self._ampm_mdl.append (["PM"])
        cell = gtk.CellRendererText ()
        self._ampm_combo.pack_start (cell, True)
        self._ampm_combo.add_attribute (cell, 'text', 0)
        self._ampm_combo.set_active (0)

        if not self._manager.has_capability ("purge"):
            self._purge_container.set_sensitive (False)
        else:
            self._purge_data_button.set_active (False)
            self._purge_spin_button.set_sensitive (False)
            self._purge_combo.set_sensitive (False)
            
            mdl = gtk.ListStore (gobject.TYPE_STRING)
            for k, v in sorted (purge_map.iteritems ()):
                mdl.append ([v])
            self._purge_combo.set_model (mdl)
            cell = gtk.CellRendererText ()
            self._purge_combo.pack_start (cell, True)
            self._purge_combo.add_attribute (cell, 'text', 0)
            self._purge_combo.set_active (2)
        
        if self._config.config_file:
            self._load ()
        
        self._window.show_all ()

    def _set_storage_device_if_available (self):
        vol_info = get_volume_info ()
        for k, v in vol_info.iteritems ():
            if v[3]:
                self._chooser_button.set_current_folder (v[0])
                return

    def _load (self):
        if not self._config.file_selection or self._config.file_selection == "all":
            self._all_data_radio.set_active (True)
        else:
            self._include_radio.set_active (True)

        if self._config.device_uuid or self._config.fuse_type:
            try:
                path = get_backup_path (self._config)
                self._chooser_button.set_current_folder (path)
            except BackupManagerException, e:
                self._set_storage_device_if_available ()
                print >> sys.stderr, str (e)
        else:
            self._set_storage_device_if_available ()
        
        if self._config.backup_schedule != None:
            self._backup_combo.set_active (self._config.backup_schedule)

        if self._config.backup_time:
            fields = self._config.backup_time.split (":")
            self._hour_spin_button.set_value (int (fields[0]))
            self._minute_spin_button.set_value (int (fields[1]))
            self._ampm_combo.set_active (0 if fields[2] == "AM" else 1)

        if self._config.includes:
            for i in self._config.includes:
                self._files_mdl.append ([i])

        if self._config.excludes:
            for e in self._config.excludes:
                self._exceptions_mdl.append ([e])
    
        if self._config.prompt_on_attach == None or self._config.prompt_on_attach == True:
            self._prompt_on_attach_button.set_active (True)
        else:
            self._prompt_on_attach_button.set_active (False)

        if self._config.purge_days:
            days = self._config.purge_days
            if days != 0:
                val = days
                units = self._config.purge_units
                
                self._purge_data_button.set_active (True)
                self._purge_container.set_sensitive (True)
                self._purge_combo.set_active (units)

                if units == PURGE_WEEKS:
                    val /= 7
                elif units == PURGE_MONTHS:
                    val /= 30

                self._purge_spin_button.set_value (int (val))

    def _save (self, button):
        self._config.file_selection = "all" if self._all_data_radio.get_active () else "specific"
        self._config.backup_schedule = self._backup_combo.get_active () 
        self._config.prompt_on_attach = self._prompt_on_attach_button.get_active () 

        if not self._purge_data_button.get_active ():
            self._config.purge_days = 0
        else:
            val = self._purge_spin_button.get_value_as_int ()
            units = self._purge_combo.get_active ()
            self._config.purge_units = units

            if units == PURGE_WEEKS:
                val *= 7
            elif units == PURGE_MONTHS:
                val *= 30
            
            self._config.purge_days = val

        fn = self._chooser_button.get_filename()
        set_backup_path (self._config, fn)
        
        self._config.backup_time = "%s:%s:%s" % (self._hour_spin_button.get_value_as_int (),
                                                 self._minute_spin_button.get_value_as_int (),
                                                 self._ampm_combo.get_active_text ())
        self._config.includes = []
        self._config.excludes = []

        if self._config.file_selection == "specific":
            for row in self._files_mdl:
                self._config.includes.append (row[0])

            for row in self._exceptions_mdl:
                self._config.excludes.append (row[0])

        if self._config.backup_schedule != SCHEDULE_NEVER:
            self._window.set_title (_("Checking disk space..."))
            self._main_content.set_sensitive (False)
            self._main_content.pack_start (self._progress_bar)
            self._progress_bar.set_text (_("Checking disk space..."))
            self._window.show_all ()
            gobject.timeout_add (500, lambda: self._progress_bar.pulse () or self._checking)
        
        t = BackupManagerThread (target=self._verify)
        t.set_callbacks (finish_cb=self._finish_cb, error_cb=self._error_cb)
        t.start ()
        self._checking = True

    def _verify (self):
        self._manager.verify_config (self._config)
        schedule_backup (self._manager, self._config)
    
    def _finish_cb (self):
        self._config.save ()
        notify_user (_("Settings Saved"), _("Your settings were saved."))
        gtk.main_quit ()
        if self._backup_now_button.get_active ():
            self._backup_now ()

    def _error_cb (self, msg):
        gtk.gdk.threads_enter ()
        self._checking = False
        self._window.set_title (_("Backup!"))
        self._main_content.set_sensitive (True)
        
        if self._config.backup_schedule != SCHEDULE_NEVER:
            self._main_content.remove (self._progress_bar)
        
        md = gtk.MessageDialog (parent=self._window, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK)
        md.set_icon (self._window.get_icon ())
        md.set_markup (_("<b>Failed to save settings:</b>\n") + msg)
        md.run ()
        md.destroy ()
        gtk.gdk.threads_leave ()
    
    def _frequency_changed (self, cmb):
        active = self._backup_combo.get_active () > 1
        self._hour_spin_button.set_sensitive (active)
        self._minute_spin_button.set_sensitive (active)
        self._ampm_combo.set_sensitive (active)

    def _files_button_clicked (self, b):
        self._select_files_window.set_transient_for (self._window)
        self._select_files_window.show_all ()
    
    def _add_file_exception (self, button):
        fc = gtk.FileChooserDialog (title=_("Add File Exception"), parent=self._window,
                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
        resp = fc.run ()
        if resp == gtk.RESPONSE_OK:
            found = False
            fn = fc.get_filename ()
            
            for row in self._exceptions_mdl:
                if row[0] == fn:
                    found = True
                    break

            if not found:
                self._exceptions_mdl.append ([fn])
        fc.destroy ()

    def _add_folder_exception (self, button):
        fc = gtk.FileChooserDialog (title=_("Add Folder Exception"), parent=self._window,
                                    action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
        resp = fc.run ()
        if resp == gtk.RESPONSE_OK:
            self._exceptions_mdl.append ([fc.get_filename ()])
        fc.destroy ()

    def _remove_exception (self, button):
        sel = self._exceptions_view.get_selection ()
        (mdl, iter) = sel.get_selected ()
        if iter:
            mdl.remove (iter)

    def _exceptions_view_selection_changed (self, view):
        sel = self._exceptions_view.get_selection ()
        (mdl, iter) = sel.get_selected ()
        self._ex_remove_button.set_sensitive (iter != None)

    def _add_file (self, button):
        fc = gtk.FileChooserDialog (title=_("Add File"), parent=self._window,
                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
        fc.set_select_multiple (True)
        resp = fc.run ()
        if resp == gtk.RESPONSE_OK:
            for fn in fc.get_filenames ():
                self._files_mdl.append ([fn])
        fc.destroy ()

    def _add_folder (self, button):
        fc = gtk.FileChooserDialog (title=_("Add Folder"), parent=self._window,
                                    action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
        fc.set_select_multiple (True)
        resp = fc.run ()
        if resp == gtk.RESPONSE_OK:
            for fn in fc.get_filenames ():
                self._files_mdl.append ([fn])
        fc.destroy ()
   
    def _remove_folder (self, button):
        sel = self._files_view.get_selection ()
        (mdl, iter) = sel.get_selected ()
        if iter:
            mdl.remove (iter)

    def _select_files_close (self, button):
        self._select_files_window.hide ()

    def _files_view_changed (self, view):
        sel = self._files_view.get_selection ()
        (mdl, iter) = sel.get_selected ()
        self._remove_button.set_sensitive (iter != None)

    def _purge_data_toggled (self, tb):
        self._purge_spin_button.set_sensitive (tb.get_active ())
        self._purge_combo.set_sensitive (tb.get_active ())
    
    def _backup_now_toggled (self, tb):
        if tb.get_active ():
            self._save_label.set_label (_("Save and Backup"))
        else:
            self._save_label.set_label (_("Save"))

    def _backup_now (self):
        pid = os.fork ()
        if pid != 0:
            os._exit (0)
        
        os.setsid ()
        os.chdir ("/")
        os.umask (0)
        
        pid = os.fork ()
        if pid != 0:
            os._exit (0)

        os.execlp ("backup-manager", "backup-manager", "--now")

def main ():
    gettext.bindtextdomain ("backup-manager", "/usr/share/locale")
    gettext.textdomain ("backup-manager")
    
    parser = OptionParser ()
    parser.add_option ("-c", "--config", dest="config",
                       default=None, help="Configuration file to use")
    (options, args) = parser.parse_args ()
    
    gtk.gdk.threads_init ()

    config_file = get_config_location ()
    config = Config ()
    
    if options.config and os.path.exists (options.config):
        config.load (options.config)
    elif os.path.exists (config_file):
        config.load (config_file)
    else:
        config.config_file = config_file

    global log
    log = get_logger (redirect=True)

    sw = SettingsWindow (config)
    gtk.main ()

if __name__ == "__main__":
    try:
        main ()
    except Exception, e:
        global log
        if not log:
            log = get_logger (redirect=True)
        
        log.error (str (e))
        sys.exit (1)
